<?php

class iGlooUser
{
    public $defaults = array(
        "level" => "",
        "first_name" => "",
        "last_name" => "",
        "name" => "",
        "email" => "",
        "password" => "",
        "settings" => "{}"
    );


    public function __construct()
    {
    }


    public function checkEmail($email)
    {
        return preg_match('/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,})$/', $email);
    }


    public function emailExists($email, $id = null)
    {
        global $wpdb;

        $sql = "SELECT * FROM `" . $this->DB_USER_TABLE . "` WHERE email = '" . addslashes($email) . "'";

        if ($id != null)
            $sql .= " AND id <> '" . addslashes($id) . "'";

        // $items = array();
        $items = $wpdb->get_results($sql, ARRAY_A);

        return $items;
    }


    public function create($params, $settings)
    {
        global $wpdb;

        if (!igCheckRequired($params, array("level", "email", "password"))) {
            return array(
                "error" => true,
                "code" => "REQUIRED"
            );
        }

        if (!$this->checkEmail($params["email"])) {
            return array(
                "error" => true,
                "code" => "EMAIL_NOT_VALID"
            );
        }

        if ($this->emailExists($params["email"])) {
            return array(
                "error" => true,
                "code" => "EMAIL_EXISTS"
            );
        }

        // Searching for the passed Level
        $level = null;
        if (!empty($settings["levels"]) && isset($params["level"])) {
            foreach ($settings["levels"] as $key => $value) {
                if ($value["id"] == $params["level"])
                    $level = $value;
            }
        }

        // Expiration Date
        $expire_at = "";
        if (!is_null($level) && isset($level["expiration"]) && $level["expiration"] != "" && isset($level["expiration_value"]) && $level["expiration_value"] != "")
            $expire_at = date('Y-m-d H:i:s', strtotime('+' . $level["expiration_value"] . ' ' . $level["expiration"]));

        $params = array_merge($this->defaults, $params);

        if ($params["name"] == "" && $params["first_name"] != "" && $params["last_name"] != "")
            $params["name"] = $params["first_name"] . " " . $params["last_name"];

        $sql = $wpdb->prepare("
            INSERT INTO " . $this->DB_USER_TABLE . "
            SET level = %d,
                first_name = %s,
                last_name = %s,
                name = %s,
                email = %s,
                password = %s,
                settings = %s,
                expire_at = %s,
                created_at = %s",
            array(
                $params["level"],
                $params["first_name"],
                $params["last_name"],
                $params["name"],
                $params["email"],
                md5($params["password"]),
                $params["settings"],
                $expire_at,
                date('Y-m-d H:i:s')
            )
        );
        $wpdb->query($sql);
        $id = $wpdb->insert_id;

        return $id;
    }




    public function extendExpiration($params, $settings)
    {
        global $wpdb;

        if (!igCheckRequired($params, array("level", "email", "password"))) {
            return array(
                "error" => true,
                "code" => "REQUIRED"
            );
        }

        $res = $this->emailExists($params["email"]);
        if (!$res) {
            return array(
                "error" => true,
                "code" => "USER_DOES_NOT_EXISTS"
            );
        }
        $user = $res[0];

        // Searching for Level
        $level = null;
        if (!empty($settings["levels"]) && isset($params["level"])) {
            foreach ($settings["levels"] as $key => $value) {
                if ($value["id"] == $params["level"])
                    $level = $value;
            }
        }

        // If Not Expiration Level
        if (is_null($level) || !isset($level["expiration"]) || $level["expiration"] == "" || !isset($level["expiration_value"]) || $level["expiration_value"] == "") {
            return array(
                "error" => true,
                "code" => "LEVEL_NOT_VALID"
            );
        }

        // Expiration Date
        if (is_null($user["expire_at"]) || $user["expire_at"] == "" || $user["expire_at"] == "0000-00-00")
            $user["expire_at"] = date('Y-M-d');
        $user["expire_at"] .= '12:00:00';
        $expire_at = date('Y-m-d H:i:s', strtotime($user["expire_at"] . ' +' . $level["expiration_value"] . ' ' . $level["expiration"]));

        // Updating DB
        $sql = $wpdb->prepare("
            UPDATE " . $this->DB_USER_TABLE . "
            SET expire_at = %s
            WHERE id = %d",
            array(
                $expire_at,
                $user["id"]
            )
        );
        $wpdb->query($sql);

        return true;
    }





    public function update($params)
    {
        global $wpdb;

        if (!igCheckRequired($params, array("id", "level", "email"))) {
            return array(
                "error" => true,
                "code" => "REQUIRED"
            );
        }

        if (!$this->checkEmail($params["email"])) {
            return array(
                "error" => true,
                "code" => "EMAIL_NOT_VALID"
            );
        }

        if ($this->emailExists($params["email"], $params["id"])) {
            return array(
                "error" => true,
                "code" => "EMAIL_EXISTS"
            );
        }

        if (isset($params["password"]) && $params["password"] != "")
            $password = ", password = md5('" . $params["password"] . "')";
        else
            $password = "";

        if (isset($params["first_name"]) && isset($params["last_name"]) != "")
            $params["name"] = $params["first_name"] . " " . $params["last_name"];

        $sql = $wpdb->prepare("
            UPDATE " . $this->DB_USER_TABLE . "
            SET level = %d,
                first_name = %s,
                last_name = %s,
                email = %s,
                settings = %s,
                expire_at = %s,
                created_at = %s
                $password
            WHERE id = %d",
            array(
                $params["level"],
                $params["first_name"],
                $params["last_name"],
                $params["email"],
                $params["settings"],
                $params["expire_at"],
                date('Y-m-d H:i:s', strtotime($params["created_at"])),
                $params["id"]
            )
        );
        $wpdb->query($sql);
    }


    public function delete($id)
    {
        global $wpdb;

        if (!isset($id))
            return;

        $sql = $wpdb->prepare("
            DELETE FROM " . $this->DB_USER_TABLE . "
            WHERE id = %d",
            array(
                $id
            )
        );
        $wpdb->query($sql);
    }


    public function getLevels()
    {
        global $wpdb;

        $sql = "SELECT level, COUNT(*) as 'num' FROM wp_igloo_users GROUP BY level";
        $items = $wpdb->get_results($sql, ARRAY_A);

        $levels = array();
        foreach ($items as $key => $value) {
            $levels[$value["level"]] = $value["num"];
        }

        return $levels;
    }


    public function getStats()
    {
        global $wpdb;

        $sql = "
            SELECT COUNT(*)
            FROM `" . $this->DB_USER_TABLE . "`";
        $all = $wpdb->get_var($sql);

        $sql = "
            SELECT COUNT(*)
            FROM `" . $this->DB_USER_TABLE . "`
            WHERE created_at >= CURDATE() - INTERVAL 1 WEEK";
        $last_week = $wpdb->get_var($sql);

        $sql = "
            SELECT COUNT(*)
            FROM `" . $this->DB_USER_TABLE . "`
            WHERE created_at >= CURDATE()";
        $today = $wpdb->get_var($sql);

        return array(
            "all" => intval($all),
            "last_week" => intval($last_week),
            "today" => intval($today)
        );
    }


    public function get($params)
    {
        global $wpdb;

        $sql = "SELECT * FROM `" . $this->DB_USER_TABLE . "` WHERE true";
        if (!is_null($params))
        {
            if (isset($params["key"]) && $params["key"] != "")
                $sql .= " AND (first_name LIKE '%" . addslashes($params["key"]) . "%' OR
                               last_name LIKE '%" . addslashes($params["key"]) . "%' OR
                               email LIKE '%" . addslashes($params["key"]) . "%')";

            if (isset($params["level"]) && $params["level"] != "")
                $sql .= " AND level = '" . addslashes($params["level"]) . "'";
        }
        $sql .= " ORDER BY id DESC";
        $items = $wpdb->get_results($sql, ARRAY_A);

        foreach ($items as $key => $value) {
            unset($items[$key]["password"]);
            unset($items[$key]["token"]);
        }

        return $items;
    }


    public function one($params)
    {
        global $wpdb;

        $sql = $wpdb->prepare("SELECT * FROM " . $this->DB_USER_TABLE . " WHERE id = %d", array($params["id"]));
        $wpdb->query($sql);
        $item = $wpdb->get_results($sql, ARRAY_A);

        return $item;
    }


    public function signup($settings, $params)
    {
        global $wpdb;

        if (!igCheckRequired($params, array("level", "email", "password"))) {
            return array(
                "error" => true,
                "code" => "REQUIRED"
            );
        }

        if (!$this->checkEmail($params["email"])) {
            return array(
                "error" => true,
                "code" => "EMAIL_NOT_VALID"
            );
        }

        if ($this->emailExists($params["email"])) {
            return array(
                "error" => true,
                "code" => "EMAIL_EXISTS"
            );
        }

        if (strlen($params["password"]) < 6) {
            return array(
                "error" => true,
                "code" => "PASSWORDS_LENGTH"
            );
        }

        if ($params["password"] != $params["password2"]) {
            return array(
                "error" => true,
                "code" => "PASSWORDS_DO_NOT_MATCH"
            );
        }

        $params = array_merge($this->defaults, $params);

        $sql = $wpdb->prepare("
            INSERT INTO " . $this->DB_USER_TABLE . "
            SET level = %d,
                first_name = %s,
                last_name = %s,
                email = %s,
                password = %s,
                settings = %s,
                created_at = %s",
            array(
                $params["level"],
                $params["first_name"],
                $params["last_name"],
                $params["email"],
                md5($params["password"]),
                $params["settings"],
                date('Y-m-d H:i:s')
            )
        );
        $wpdb->query($sql);
        $id = $wpdb->insert_id;

        // Email Body
        $message = $settings["signup_email_body"];
        $message = str_replace("{{email}}", $params["email"], $message);
        $message = str_replace("{{password}}", $params["password"], $message);
        $message = str_replace("{{login_url}}", $settings["site_url"] . "/" . $settings["login_page_url"], $message);

        // Send Email
        $to = $params["email"];
        $subject = $settings["signup_email_subject"];
        $headers = "From: " . $settings["email_sender_name"] . " <" . $settings["email_sender"] . ">\r\n";
        $headers .= "MIME-Version: 1.0\r\n";
        $headers .= "Content-Type: text/html; charset=UTF-8\r\n";
        mail($to, $subject, nl2br($message), $headers);

        return $id;
    }



    public function login($params, $settings)
    {
        global $wpdb;

        $sql = $wpdb->prepare("
            SELECT *
            FROM " . $this->DB_USER_TABLE . "
            WHERE email = %s AND
                  password = %s",
            array(
                $params["email"],
                md5($params["password"]),
            )
        );

        $wpdb->query($sql);
        $item = $wpdb->get_results($sql, ARRAY_A);

        if (count($item) > 0)
        {
            // Checking Expiration
            if (isset($item[0]["expire_at"]) && $item[0]["expire_at"] != "" && $item[0]["expire_at"] != "0000-00-00")
            {
                if ($item[0]["expire_at"] . " 23:59:59" < date("Y-m-d H:i:s"))
                    return "Your plan has expired";
            }

            if (!isset($item[0]["created_at"]) || $item[0]["created_at"] == "") {
                $sql = $wpdb->prepare("
            UPDATE " . $this->DB_USER_TABLE . "
            SET created_at = %s
            WHERE id = %d",
                    [date('Y-m-d H:i:s'), $item[0]["id"]]
                );
                $wpdb->query($sql);
            }

            // Writing Token to Database
            $item[0]["token"] = $token = md5(uniqid($params["email"], true));
            $sql = $wpdb->prepare("
                UPDATE " . $this->DB_USER_TABLE . "
                SET token = %s
                WHERE id = %d",
                array(
                    $token,
                    $item[0]["id"]
                )
            );
            $wpdb->query($sql);

            // Creating the Cookie
            setcookie("igloo_token", $token, time() + (86400 * 14), "/"); // 14 days

            return $item[0];
        }

        return "Invalid email or password";
    }





    public function lostPassword($params, $settings)
    {
        global $wpdb;

        $sql = $wpdb->prepare("
            SELECT *
            FROM " . $this->DB_USER_TABLE . "
            WHERE email = %s",
            array(
                $params["email"],
            )
        );

        $wpdb->query($sql);
        $item = $wpdb->get_results($sql, ARRAY_A);

        if (count($item) > 0)
        {
            // Checking Expiration
            if (isset($item[0]["expire_at"]) && $item[0]["expire_at"] != "" && $item[0]["expire_at"] != "0000-00-00")
            {
                if ($item[0]["expire_at"] . " 23:59:59" < date("Y-m-d H:i:s"))
                    return false;
            }

            // Writing Reset Code to Database
            $item[0]["reset_code"] = $reset_code = igGenerateAlpha(10);
            $sql = $wpdb->prepare("
                UPDATE " . $this->DB_USER_TABLE . "
                SET reset_code = %s
                WHERE id = %d",
                array(
                    $reset_code,
                    $item[0]["id"]
                )
            );
            $wpdb->query($sql);

            // Url
            $url = $settings["site_url"] . "?iglogin=1&reset=1&email=" . $item[0]["email"] . "&code=" . $reset_code;

            // Email Body
            $message = $settings["password_reset_email_body"];
            $message = str_replace("{{url}}", $url, $message);
            $message = str_replace("{{login_url}}", $settings["site_url"] . "?iglogin=1", $message);

            // Send Email
            $to = $item[0]["email"];
            $subject = $settings["password_reset_email_subject"];
            $headers = "From: " . $settings["email_sender_name"] . " <" . $settings["email_sender"] . ">\r\n";
            $headers .= "MIME-Version: 1.0\r\n";
            $headers .= "Content-Type: text/html; charset=UTF-8\r\n";
            mail($to, $subject, nl2br($message), $headers);

            return $item[0];
        }

        return false;
    }



    public function resetPassword($params, $settings)
    {
        global $wpdb;

        $sql = $wpdb->prepare("
            SELECT *
            FROM " . $this->DB_USER_TABLE . "
            WHERE email = %s AND
                  reset_code = %s",
            array(
                $params["email"],
                $params["code"],
            )
        );

        $wpdb->query($sql);
        $item = $wpdb->get_results($sql, ARRAY_A);

        if (count($item) > 0)
        {
            // Checking Expiration
            if (isset($item[0]["expire_at"]) && $item[0]["expire_at"] != "" && $item[0]["expire_at"] != "0000-00-00")
            {
                if ($item[0]["expire_at"] . " 23:59:59" < date("Y-m-d H:i:s"))
                    return false;
            }

            // Writing new password to database
            $item[0]["token"] = $token = md5(uniqid($params["email"], true));
            $sql = $wpdb->prepare("
                UPDATE " . $this->DB_USER_TABLE . "
                SET password = %s
                WHERE id = %d",
                array(
                    md5($params["password"]),
                    $item[0]["id"]
                )
            );
            $wpdb->query($sql);

            return $item[0];
        }

        return false;
    }



    public function checkToken($params)
    {
        global $wpdb;

        $sql = $wpdb->prepare("SELECT * FROM " . $this->DB_USER_TABLE . " WHERE token = %s", array($params["token"]));
        $wpdb->query($sql);
        $items = $wpdb->get_results($sql, ARRAY_A);

        if (count($items) > 0)
        {
            // Checking Expiration
            if (isset($items[0]["expire_at"]) && $items[0]["expire_at"] != "" && $items[0]["expire_at"] != "0000-00-00")
            {
                if ($items[0]["expire_at"] . " 23:59:59" < date("Y-m-d H:i:s"))
                    return false;
            }

            return $items[0];
        }

        return false;
    }


    public function logout()
    {
        if (isset($_COOKIE['igloo_token'])) {
            setcookie("igloo_token", null, -1, '/');
        }
    }

}

?>
